/*
 * This file is part of SCHUR.
 *
 * SCHUR - an interactive program for calculating properties of Lie
 * groups and symmetric functions.
 * Copyright (C) 2006  Franck BUTELLE, Frédéric Toumazet
 * 
 * SCHUR is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * SCHUR is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SCHUR; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include "standard.h"

#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <strings.h>


#include "dim.h"
#include "type.h"
#include "var.h"
#include "FillCommandTab.h"

#include "../config.h"
#ifdef HAVE_LIBREADLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

static int
cmpstringp(const void *p1, const void *p2)
{
  /* The actual arguments to this function are "pointers to
   * pointers to char", but strcasecmp() arguments are "pointers
   * to char", hence the following cast plus dereference */

   return strcasecmp(* (char * const *) p1, * (char * const *) p2);
}

int 
fillCommandTab(void)
{
  struct dirent *d;
  DIR *dirstream;
  char helpdir[MAXSTRING], *pathvar;
  
  if ((pathvar = getenv ("SCHURLIB")) != NULL)
	  sprintf (helpdir, "%s/help", pathvar);
  else
	  sprintf (helpdir, "%s/help", DATADIR);

  dirstream = opendir (helpdir);
  
  int i=0;
  
  if (dirstream == NULL) {
	perror("Error opening help files directory\n");
        exit(1);
  }

  while ( (d=readdir(dirstream)) != NULL) {
	if (d->d_name[0] >= 'A' && d->d_name[0]<='Z' ) {
	  //printf("%s\n",d->d_name); 
          if (i >= MAXNBCOMMANDS) {
		fprintf(stderr, "ERROR: Table of commands is full !\n");
		exit(1);
          }
          CommandTab[i++] = strdup(d->d_name);
        }
  }

  qsort((void *)CommandTab, i, sizeof (char *), cmpstringp);

  return(i);
}

/* Generator function for command completion.  STATE lets us
 * know whether to start from scratch; without any state
 * (i.e. STATE == 0), then we start at the top of the list. */
char *
command_generator (const char *text, int state)
{
	static int list_index, len;
	char *name;

  /* If this is a new word to complete, initialize now.  This
	* includes saving the length of TEXT for efficiency, and
  * initializing the index variable to 0. */
	if (!state)
	{
		list_index = 0;
		len = strlen (text);
	}

  /* Return the next name which partially matches from the
  * command list. */
	while ( (name = CommandTab[list_index]) )
	{
		list_index++;

		if (strncasecmp (name, text, len) == 0)
			return (strdup (name));
	}

	/* If no names matched, then return NULL. */
	return ((char *) NULL);
}

/* Attempt to complete on the contents of TEXT.  START and END
 * bound the region of rl_line_buffer that contains the word to
 * complete.  TEXT is the word to complete.  We can use the entire
 * contents of rl_line_buffer in case we want to do some simple
 * parsing.  Return the array of matches, or NULL if there aren't any. */
char **
Command_completion (const char *text, int start, int end)
{
	char **matches;

	matches = (char **) NULL;
#ifdef HAVE_LIBREADLINE
	matches = rl_completion_matches (text, command_generator);
#endif
	return (matches);
}

void
initialise_readline (void)
{
#ifdef HAVE_LIBREADLINE
  /* Allow conditional parsing of the ~/.inputrc file. */
  rl_readline_name = "fillCommand";

  /* Tell the completer that we want a crack first. */
  rl_attempted_completion_function = Command_completion;
#endif
}

/*
char *
stripwhite (char *string)
{
  register char *s, *t;

  for (s = string; whitespace (*s); s++)
    ;

  if (*s == 0)
    return (s);

  t = s + strlen (s) - 1;
  while (t > s && whitespace (*t))
    t--;
  *++t = '\0';

  return s;
}


void
execute_line (char *line)
{
	printf("exec:%s\n",line);
}


int 
main (int argc, char *argv[]) {
  int i, done=0, NbCommands;
  char *line, *s;
  
  initialize_readline ();	* Bind our completer. *
  
  NbCommands = fillCommandTab();
  for (i=0; i<NbCommands;i++)
     printf("%s ", CommandTab[i]);
  printf (" - %d commands.\n", NbCommands);
  
  for (; done == 0;)
    {
      line = readline ("Schur: ");

      if (!line)
	break;

      s = stripwhite (line);

      if (*s)
	{
	  add_history (s);
	  execute_line (s);
	}
      free (line);
    }
  return 0;
}
*/
