/*
 * This file is part of SCHUR.
 *
 * SCHUR - an interactive program for calculating properties of Lie
 * groups and symmetric functions.
 * Copyright (C) 2006  Franck BUTELLE, Frédéric Toumazet
 * 
 * SCHUR is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * SCHUR is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SCHUR; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

# include <stdbool.h>
# include "dim.h"
# include "define.h"
# include "type.h"
# include "var.h"
# include "r.h"
# include "utils.h"
# include "write.h"

#define RESTORE_CURSOR_POSITION()   printf("[17D[1A                     [21D")

/* return the frame in s in display format (TeX or not) and the length of the
 * resulting string */
int
wrtfrme2 (char *s, frame x)
{
  int i, k, m, t;
  register int j;
  char *saveptr = s;

  t = maxdim - 2;
  while ((x.A[t] == 0) && (t > 0))
    t = t - 1;
  if (!pow_note)
    {
      i = 2;
      s += sprintf (s, "%1d", x.A[1]);
      if (((x.A[1] > 9) && tex))
	s += sprintf (s, "\\");
      else
	s += sprintf (s, " ");
      while ((i <= t))
	{
	  s += sprintf (s, "%1d", x.A[i]);
	  if (((abs (x.A[i]) > 9) && tex))
	    s += sprintf (s, "\\");
	  else
	    s += sprintf (s, " ");

	  i = i + 1;
	}
    }
  else
    {
      i = 1;
      m = 1;
      while ((m <= t) && (i < maxdim - 2))
	{
	  j = x.A[i];
	  k = 1;
	  do
	    {
	      i = i + 1;
	      if ((x.A[i] == j))
		k = k + 1;
	    }
	  while (!((x.A[i] != j) || (i == maxdim - 2)));
	  if (k > cutoff)
	    {
	      if (((m > 1) && (abs (x.A[m]) > 9)))
		if (tex)
		  s += sprintf (s, "\\");
		else
		  s += sprintf (s, " ");

	      if (tex)
		{
		  s += sprintf (s, "%1d^", x.A[m]);
		  if ((k > 9))
		    s += sprintf (s, "{%1d}\\", k);
		  else
		    s += sprintf (s, "%1d", k);
		}
	      else
		{
		  s += sprintf (s, "%1d^%1d", x.A[m], k);
		  if (i <= t)
		    s += sprintf (s, " ");
		}
	    }
	  else
	    for (j = 1; j <= k; j++)
	      {
		if (abs (x.A[m]) > 9)
		  {
		    if (tex)
		      s += sprintf (s, "%1d\\", x.A[m]);
		    else
		      s += sprintf (s, "%2d ", x.A[m]);
		    if ((x.A[m] < 0))
		      if (tex)
			s += sprintf (s, "\\");
		      else
			s += sprintf (s, " ");
		  }
		else
		  s += sprintf (s, "%1d", x.A[m]);
	      }
	  m = i;
	}
      if (i == 1)
	s += sprintf (s, "0");
    }
  return (s - saveptr);
}

void
wrttlst2 (text * fyle, int *qq, int col, char a, char b, termptr list,
	  bool vdu)
{
  bool startx;
  int line = 1;
  char key = ' ';
  termptr ptr;
  char intertotal[500];
  char *inter;

  if (list == NULL && !qtest)
    {
      fprintf (fyle->fp, "zero\n");
      return;
    }
  startx = true;
  while ((list != NULL) && (key != 'q'))
    {
      ptr = list;
      inter = intertotal;
      if (!startx)
	inter += sprintf (inter, " ");
      if (ptr->mult == -1)
	inter += sprintf (inter, "-");
      if (!startx && ptr->mult >= 1)
	inter += sprintf (inter, "+");
      if (abs (ptr->mult) != 1)
	inter += sprintf (inter, "%d", ptr->mult);
      if ((qfn == true) && (pfn == false))
	inter += sprintf (inter, "Q_");
      else if ((pfn == true) && (qfn == true))
	inter += sprintf (inter, "P_");
      else if ((mmono == true))
	inter += sprintf (inter, "m_");
      else if ((forg == true))
	inter += sprintf (inter, "f_");
      else if ((homo == true))
	inter += sprintf (inter, "h_");
      else if ((psum == true))
	inter += sprintf (inter, "p_");
      else if ((elem == true))
	inter += sprintf (inter, "e_");
      inter += sprintf (inter, "%c", a);
      inter += wrtfrme2 (inter, ptr->val);
      startx = false;
      inter += sprintf (inter, "%c", b);
      if ((ptr->slab == '#'))
	sprintf (inter, "#");
      if (*qq + strlen (intertotal) > tcol)
	{
	  fprintf (fyle->fp, "\n");
	  *qq = 0;
	  if (vdu && more)
	    {
	      line = line + 1;

	      if (line == tlines-1)
		{
		  line = 1;
		  if (more_Question () == 'q')
		    {
		  //    fprintf (fyle->fp, "\n");
		      return;
		    }
		  else
		    RESTORE_CURSOR_POSITION ();
		}
	    }
	}
      *qq += fprintf (fyle->fp, intertotal);
      list = ptr->next;
    }
}


void
writer2 (text * fyle, int *qq, int col, char a, char b, caystype cays,
	 ocharptr list, bool vdu)
{
  char key, spinchar;
  bool startx;
  int i, line, k, m, z = 0,	// modified by FB, was unitialized
    tcolumns, sline;
  register int l;
  ocharptr ptrlist;
  caystype caystmp;
  char intertotal[500];
  char *s;

  line = 1;
  key = '?';
  spinchar = 's';
  sline = 1;
  tcolumns = 0;
  if (list == NULL)
    {
      *qq += fprintf ((*fyle).fp, "zero");
      return;
    }
  startx = true;
  ptrlist = list;
  while ((ptrlist != NULL) && (key != esc))
    {
      s = intertotal;

      if (startx)
	startx = false;
      else
	{
	  s += sprintf (s, " ");
	  if (ptrlist->mult > 0)
	    s += sprintf (s, "+");
	}
      if (ptrlist->mult == -1)
	s += sprintf (s, "-");
      else if (ptrlist->mult != 1)
	s += sprintf (s, "%d", ptrlist->mult);
      switch ((int) (currgrp.A[1 - 1].name))
	{
	case sn:
	  if (nreduce)
	    {
	      a = '<';
	      b = '>';
	    }
	  else
	    {
	      a = '{';
	      b = '}';
	    }
	  break;
	case an:
	  if (nreduce)
	    {
	      a = '<';
	      b = '>';
	    }
	  else
	    {
	      a = '{';
	      b = '}';
	    }
	  break;
	case sung:
	case un:
	case unm:
	case sunm:
	case unc:
	  a = '{';
	  b = '}';
	  break;
	case son:
	case on:
	case sonc:
	  a = '[';
	  b = ']';
	  break;
	case spn:
	  a = '<';
	  b = '>';
	  break;
	case spnc:
	case mp:
	  a = '<';
	  b = '>';
	  break;
	case ospnm:
	  a = '[';
	  b = '>';
	  break;
	case e6:
	case e7:
	case e8:
	case f4:
	case g2:
	case en:
	case l168:
	  a = '(';
	  b = ')';
	  break;
	case nill:
	  warn ("no group set;", cr);
	  break;
	default:
	  Caseerror (Line);
	}
      if (currgrp.A[1 - 1].name == e6)
	caystmp = pe6;
      else
	caystmp = norm;
      s += sprintf (s, "%c", a);
      if ((currgrp.A[1 - 1].name == unc))
	s += sprintf (s, "(");
      if (ptrlist->spin)
	{
	  s += sprintf (s, "%c", spinchar);
	  if (((currgrp.A[1 - 1].name != spnc)
	       && (currgrp.A[1 - 1].name != mp)
	       && (currgrp.A[1 - 1].name != sonc)))
	    s += sprintf (s, ";");
	}
      if ((ptrlist->C6_double && (currgrp.A[1 - 1].name != spnc)
	   && (currgrp.A[1 - 1].name != mp)
	   && (currgrp.A[1 - 1].name != sonc)))
	{
	  s += wrtfrme2 (s, ptrlist->conval);
	  s += sprintf (s, ";");
	}
      if ((currgrp.A[1 - 1].name == spnc) || (currgrp.A[1 - 1].name == mp)
	  || (currgrp.A[1 - 1].name == sonc))
	s += sprintf (s, "%d(", ptrlist->val.A[maxdim]);

      if (caystmp == pe6)
	{
	  s += sprintf (s, "%d:", ptrlist->val.A[1]);
	  if (ptrlist->val.A[2] == 0)
	    {
	      s += sprintf (s, "0");
	      i = 3;
	    }
	  else
	    i = 2;
	  if (!pow_note)
	    {
	      while (ptrlist->val.A[i] != 0)
		{
		  s += sprintf (s, "%1d", ptrlist->val.A[i]);
		  i = i + 1;
		}
	    }
	  else
	    {
	      if ((group == g2))
		if ((racah == true))
		  {
		    z = ptrlist->val.A[1];
		    ptrlist->val.A[1] = z - ptrlist->val.A[2];
		  }
	      m = i;
	      while (ptrlist->val.A[m] != 0)
		{
		  l = ptrlist->val.A[i];
		  i = i + 1;
		  k = 1;
		  while (ptrlist->val.A[i] == l)
		    {
		      i = i + 1;
		      k = k + 1;
		    }
		  if (k > cutoff)
		    {
		      if (ptrlist->val.A[m] > 9)
			s += sprintf (s, " ");
		      s += sprintf (s, "%1d^%1d", ptrlist->val.A[m], k);
		    }
		  else
		    for (l = 1; l <= k; l++)
		      {
			if (ptrlist->val.A[m] > 9)
			  s += sprintf (s, "%d ", ptrlist->val.A[m]);
			else
			  s += sprintf (s, "%d", ptrlist->val.A[m]);
		      }
		  m = i;
		}
	      if ((group == g2))
		if ((racah == true))
		  ptrlist->val.A[1] = z;
	    }
	}
      else
	if ((currgrp.A[1 - 1].name != spnc)
	    && (currgrp.A[1 - 1].name != mp)
	    && (currgrp.A[1 - 1].name != sonc))
	s += wrtfrme2 (s, ptrlist->val);
      if ((currgrp.A[1 - 1].name == spnc) || (currgrp.A[1 - 1].name == mp))
	{
	  s += wrtfrme2 (s, ptrlist->val);
	  s += sprintf (s, ")>");
	}
      if ((currgrp.A[1 - 1].name == spnc))
	{
	  if (ptrlist->lab != ' ')
	    s += sprintf (s, "#");
	}
      if ((currgrp.A[1 - 1].name == sonc))
	{
	  s += wrtfrme2 (s, ptrlist->val);
	  s += sprintf (s, ")]");
	}
      if ((currgrp.A[1 - 1].name == unc))
	s += sprintf (s, ")");
      if ((currgrp.A[1 - 1].name != spnc) && (currgrp.A[1 - 1].name != mp)
	  && (currgrp.A[1 - 1].name != sonc))
	{
	  s += sprintf (s, "%c", b);
	  if (ptrlist->lab != ' ')
	    s += sprintf (s, "%c", ptrlist->lab);
	}
      if (*qq + strlen (intertotal) > tcol)
	{
	  fprintf (fyle->fp, "\n");
	  *qq = 0;
	  if (vdu && more)
	    {
	      line = line + 1;

	      if (line == tlines -1)
		{
		  line = 1;
		  if (more_Question () == 'q')
		    {
		      return;
		    }
		  else
		    RESTORE_CURSOR_POSITION ();
		}
	    }
	}
      *qq += fprintf (fyle->fp, intertotal);

      ptrlist = ptrlist->next;
    }
}

void
writeprod2 (text * fyle, int *qq, prodtype pr, int col, bool vdu)
{
  bool startx, spinc;
  int z = 0,			// modified by FB, was uninitialized
    i, line, k, m, sline;
  register int l;
  register int j;
  caystype cays;
  prodtype top, ptr;
  char a = '{', b = '}',	// modified by FB, was uninitialized
    spinchar, key;
  char intertotal[500];
  char *s;

  line = 1;
  key = '?';
  spinchar = 's';
  sline = 1;
  top = pr;
  if (pr == NULL)
    {
      *qq += fprintf ((*fyle).fp, "zero");
      return;
    }
  startx = true;
  ptr = pr;
  while ((ptr != NULL) && (key != esc))
    {
      s = intertotal;

      if (startx)
	startx = false;
      else
	{
	  s += sprintf (s, " ");
	  if (ptr->mult > 0)
	    s += sprintf (s, "+");
	}
      if (ptr->mult == -1)
	s += sprintf (s, "-");
      else if (ptr->mult != 1)
	s += sprintf (s, "%d", ptr->mult);

      for (j = 1; j <= nprod; j++)
	{
	  switch ((int) (currgrp.A[j - 1].name))
	    {
	    case sn:
	      if (nreduce)
		{
		  a = '<';
		  b = '>';
		}
	      else
		{
		  a = '{';
		  b = '}';
		}
	      break;
	    case an:
	      if (nreduce)
		{
		  a = '<';
		  b = '>';
		}
	      else
		{
		  a = '{';
		  b = '}';
		}
	      break;
	    case sung:
	    case un:
	    case unm:
	    case sunm:
	    case unc:
	      a = '{';
	      b = '}';
	      break;
	    case son:
	    case on:
	    case sonc:
	      a = '[';
	      b = ']';
	      break;
	    case spn:
	      a = '<';
	      b = '>';
	      break;
	    case spnc:
	    case mp:
	      a = '<';
	      b = '>';
	      spinc = ptr->prods.A[j - 1]->spin;
	      break;
	    case ospnm:
	      a = '[';
	      b = '>';
	      break;
	    case e6:
	    case e7:
	    case e8:
	    case f4:
	    case g2:
	    case en:
	    case l168:
	      a = '(';
	      b = ')';
	      break;
	    case nill:
	      warn ("no group set;", cr);
	      break;
	    default:
	      Caseerror (Line);
	    }
	  if (currgrp.A[j - 1].name == e6)
	    cays = pe6;
	  else
	    cays = norm;
	  s += sprintf (s, "%c", a);
	  if (currgrp.A[j - 1].name == unc)
	    s += sprintf (s, "(");
	  if (ptr->prods.A[j - 1]->spin)
	    {
	      s += sprintf (s, "%c", spinchar);
	      if (((currgrp.A[j - 1].name != spnc)
		   && (currgrp.A[1 - 1].name != mp)
		   && (currgrp.A[1 - 1].name != sonc)))
		s += sprintf (s, ";");
	    }
	  if (ptr->prods.A[j - 1]->C6_double
	      && (currgrp.A[j - 1].name != spnc)
	      && (currgrp.A[1 - 1].name != mp)
	      && (currgrp.A[1 - 1].name != sonc))
	    {
	      s += wrtfrme2 (s, ptr->prods.A[j - 1]->conval);
	      s += sprintf (s, ";");
	    }
	  if (((currgrp.A[j - 1].name == spnc)
	       || (currgrp.A[1 - 1].name == mp)
	       || (currgrp.A[1 - 1].name == sonc)))
	    {
	      s += sprintf (s, "%d(", ptr->prods.A[j - 1]->val.A[maxdim]);
	    }
	  if (cays == pe6)
	    {
	      s += sprintf (s, "%d:", ptr->prods.A[j - 1]->val.A[1]);
	      if (ptr->prods.A[j - 1]->val.A[2] == 0)
		{
		  s += sprintf (s, "0");
		  i = 3;
		}
	      else
		i = 2;
	      if (!pow_note)
		{
		  while (ptr->prods.A[j - 1]->val.A[i] != 0)
		    {
		      s += sprintf (s, "%d", ptr->prods.A[j - 1]->val.A[i]);
		      i = i + 1;
		    }
		}
	      else
		{
		  register ocharptr W33 = ptr->prods.A[j - 1];

		  if ((group == g2))
		    if ((racah == true))
		      {
			z = W33->val.A[1];
			W33->val.A[1] = z - W33->val.A[2];
		      }
		  m = i;
		  while ((W33->val.A[m] != 0))
		    {
		      l = W33->val.A[i];
		      i = i + 1;
		      k = 1;
		      while (W33->val.A[i] == l)
			{
			  i = i + 1;
			  k = k + 1;
			}
		      if (k > cutoff)
			{
			  if (W33->val.A[m] > 9)
			    s += sprintf (s, " ");
			  s += sprintf (s, "%d^%d", W33->val.A[m], k);
			}
		      else
			for (l = 1; l <= k; l++)
			  {
			    if (W33->val.A[m] > 9)
			      s += sprintf (s, "%d ", W33->val.A[m]);
			    else
			      s += sprintf (s, "%d", W33->val.A[m]);
			  }
		      m = i;
		    }
		  if ((group == g2))
		    if ((racah == true))
		      W33->val.A[1] = z;
		}
	    }
	  else
	    if (((currgrp.A[j - 1].name != spnc)
		 && (currgrp.A[j - 1].name != mp)
		 && (currgrp.A[1 - 1].name != sonc)))
	    s += wrtfrme2 (s, ptr->prods.A[j - 1]->val);
	  if ((currgrp.A[j - 1].name == spnc)
	      || (currgrp.A[j - 1].name == mp))
	    {
	      s += wrtfrme2 (s, ptr->prods.A[j - 1]->val);
	      s += sprintf (s, ")>");
	    }
	  if ((currgrp.A[1 - 1].name == sonc))
	    {
	      s += wrtfrme2 (s, ptr->prods.A[j - 1]->val);
	      s += sprintf (s, ")]");
	    }
	  if (currgrp.A[j - 1].name == unc)
	    s += sprintf (s, ")");
	  if (((currgrp.A[j - 1].name != spnc)
	       && (currgrp.A[j - 1].name != mp)
	       && (currgrp.A[1 - 1].name != sonc)))
	    {
	      s += sprintf (s, "%c", b);
	      if (ptr->prods.A[j - 1]->lab != ' ')
		s += sprintf (s, "%c", ptr->prods.A[j - 1]->lab);
	    }
	}
      if (*qq + strlen (intertotal) > tcol)
	{
	  fprintf (fyle->fp, "\n");
	  *qq = 0;
	  if (vdu && more)
	    {
	      line = line + 1;
	      if (line == tlines -1)
		{
		  line = 1;

		  if (more_Question () == 'q')
		    {
		      return;
		    }
		  else
		    RESTORE_CURSOR_POSITION ();
		}
	    }
	}
      *qq += fprintf (fyle->fp, intertotal);

      ptr = ptr->next;
    }
}

void
writepoly (lbframe apoly)
{
  register int p;
  bool notfirst = false;	// if the poly is equals to 0, notfirst will stay at false.
  char intertotal[maxdim], *s;	// used for the display width mgmt.
  char var = 'q';
  int index;

  //Putchr ('\n', output);

  s = intertotal;
  if (apoly.A[0] != 0)
    {				// degree 0
      if (apoly.A[0] == 1)
	s += sprintf (s, "1");
      else
	s += sprintf (s, "%d", apoly.A[0]);
      notfirst = true;
    }

  if (apoly.A[1] != 0)
    {				// degree 1
      if (notfirst)
	{
	  if (apoly.A[1] == 1)
	    s += sprintf (s, " +%c", var);
	  else if (apoly.A[1] == -1)
	    s += sprintf (s, " -%c", var);
	  else
	    s += sprintf (s, " %+d%c", apoly.A[1], var);
	}
      else
	{			// first term
	  if (apoly.A[1] == 1)
	    s += sprintf (s, "%c", var);
	  else if (apoly.A[1] == -1)
	    s += sprintf (s, "-%c", var);
	  else
	    s += sprintf (s, "%d%c", apoly.A[1], var);
	}
      notfirst = true;
    }

  print ("%s", intertotal);
  index = strlen (intertotal);
  for (p = 2; p <= setlimit; p++)	// degrees>1
    {
      s = intertotal;

      if (apoly.A[p] != 0)
	{
	  if (notfirst)
	    {
	      if (apoly.A[p] == 1)
		s += sprintf (s, " +%c^%d", var, p);
	      else if (apoly.A[p] == -1)
		s += sprintf (s, " -%c^%d", var, p);
	      else
		s += sprintf (s, " %+d%c^%d", apoly.A[p], var, p);
	    }
	  else
	    {
	      if (apoly.A[p] == 1)
		s += sprintf (s, "%c^%d", var, p);
	      else if (apoly.A[p] == -1)
		s += sprintf (s, "-%c^%d", var, p);
	      else
		s += sprintf (s, "%d%c^%d", apoly.A[p], var, p);
	    }
	  notfirst = true;
	  index += strlen (intertotal);
	  if (index > tcol)
	    {
	      print ("\n%s", intertotal);
	      index = strlen (intertotal);
	    }
	  else
	    print ("%s", intertotal);
	}
    }
  if (!notfirst)		// the poly is equals to 0
    print ("0");

  print ("\n");
}
